@@ -13,9 +13,9 @@
 
 """Network action implementations"""
 
-from openstackclient.common import command
-from openstackclient.common import exceptions
-from openstackclient.common import utils
+from osc_lib.command import command
+from osc_lib import utils
+
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 from openstackclient.network import common
@@ -32,7 +32,7 @@ def _format_router_external(item):
 _formatters = {
     'subnets': utils.format_list,
     'admin_state_up': _format_admin_state,
-    'router_external': _format_router_external,
+    'router:external': _format_router_external,
     'availability_zones': utils.format_list,
     'availability_zone_hints': utils.format_list,
 }
@@ -43,9 +43,6 @@ def _get_columns(item):
     if 'tenant_id' in columns:
         columns.remove('tenant_id')
         columns.append('project_id')
-    if 'router:external' in columns:
-        columns.remove('router:external')
-        columns.append('router_external')
     return tuple(sorted(columns))
 
 
@@ -61,6 +58,10 @@ def _get_attrs(client_manager, parsed_args):
         attrs['shared'] = True
     if parsed_args.no_share:
         attrs['shared'] = False
+    if parsed_args.enable_port_security:
+        attrs['port_security_enabled'] = True
+    if parsed_args.disable_port_security:
+        attrs['port_security_enabled'] = False
 
     # "network set" command doesn't support setting project.
     if 'project' in parsed_args and parsed_args.project is not None:
@@ -77,6 +78,10 @@ def _get_attrs(client_manager, parsed_args):
        parsed_args.availability_zone_hints is not None:
         attrs['availability_zone_hints'] = parsed_args.availability_zone_hints
 
+    # set description
+    if parsed_args.description:
+        attrs['description'] = parsed_args.description
+
     # update_external_network_options
     if parsed_args.internal:
         attrs['router:external'] = False
@@ -93,19 +98,25 @@ def _get_attrs(client_manager, parsed_args):
         attrs['provider:physical_network'] = parsed_args.physical_network
     if parsed_args.segmentation_id:
         attrs['provider:segmentation_id'] = parsed_args.segmentation_id
+    # Update VLAN Transparency for networks
+    if parsed_args.transparent_vlan:
+        attrs['vlan_transparent'] = True
+    if parsed_args.no_transparent_vlan:
+        attrs['vlan_transparent'] = False
     return attrs
 
 
-def _add_provider_network_options(parser):
-    # Add provider network options
+def _add_additional_network_options(parser):
+    # Add additional network options
+
     parser.add_argument(
         '--provider-network-type',
         metavar='<provider-network-type>',
-        choices=['flat', 'gre', 'local',
+        choices=['flat', 'geneve', 'gre', 'local',
                  'vlan', 'vxlan'],
         help=_("The physical mechanism by which the virtual network "
                "is implemented. The supported options are: "
-               "flat, gre, local, vlan, vxlan"))
+               "flat, geneve, gre, local, vlan, vxlan."))
     parser.add_argument(
         '--provider-physical-network',
         metavar='<provider-physical-network>',
@@ -116,8 +127,18 @@ def _add_provider_network_options(parser):
         '--provider-segment',
         metavar='<provider-segment>',
         dest='segmentation_id',
-        help=_("VLAN ID for VLAN networks or Tunnel ID for GRE/VXLAN "
-               "networks"))
+        help=_("VLAN ID for VLAN networks or Tunnel ID for "
+               "GENEVE/GRE/VXLAN networks"))
+
+    vlan_transparent_grp = parser.add_mutually_exclusive_group()
+    vlan_transparent_grp.add_argument(
+        '--transparent-vlan',
+        action='store_true',
+        help=_("Make the network VLAN transparent"))
+    vlan_transparent_grp.add_argument(
+        '--no-transparent-vlan',
+        action='store_true',
+        help=_("Do not make the network VLAN transparent"))
 
 
 def _get_attrs_compute(client_manager, parsed_args):
@@ -174,6 +195,11 @@ class CreateNetwork(common.NetworkAndComputeShowOne):
             metavar='<project>',
             help=_("Owner's project (name or ID)")
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set network description")
+        )
         identity_common.add_project_domain_option_to_parser(parser)
         parser.add_argument(
             '--availability-zone-hint',
@@ -184,6 +210,19 @@ class CreateNetwork(common.NetworkAndComputeShowOne):
                    "(Network Availability Zone extension required, "
                    "repeat option to set multiple availability zones)")
         )
+        port_security_group = parser.add_mutually_exclusive_group()
+        port_security_group.add_argument(
+            '--enable-port-security',
+            action='store_true',
+            help=_("Enable port security by default for ports created on "
+                   "this network (default)")
+        )
+        port_security_group.add_argument(
+            '--disable-port-security',
+            action='store_true',
+            help=_("Disable port security by default for ports created on "
+                   "this network")
+        )
         external_router_grp = parser.add_mutually_exclusive_group()
         external_router_grp.add_argument(
             '--external',
@@ -206,10 +245,10 @@ class CreateNetwork(common.NetworkAndComputeShowOne):
         default_router_grp.add_argument(
             '--no-default',
             action='store_true',
-            help=_("Do not use the network as the default external network. "
+            help=_("Do not use the network as the default external network "
                    "(default)")
         )
-        _add_provider_network_options(parser)
+        _add_additional_network_options(parser)
         return parser
 
     def update_parser_compute(self, parser):
@@ -235,51 +274,96 @@ class CreateNetwork(common.NetworkAndComputeShowOne):
         return (columns, data)
 
 
-class DeleteNetwork(common.NetworkAndComputeCommand):
+class DeleteNetwork(common.NetworkAndComputeDelete):
     """Delete network(s)"""
 
+    # Used by base class to find resources in parsed_args.
+    resource = 'network'
+    r = None
+
     def update_parser_common(self, parser):
         parser.add_argument(
             'network',
             metavar="<network>",
             nargs="+",
-            help=("Network(s) to delete (name or ID)")
+            help=_("Network(s) to delete (name or ID)")
         )
+
         return parser
 
     def take_action_network(self, client, parsed_args):
-        for network in parsed_args.network:
-            obj = client.find_network(network)
-            client.delete_network(obj)
+        obj = client.find_network(self.r, ignore_missing=False)
+        client.delete_network(obj)
 
     def take_action_compute(self, client, parsed_args):
-        for network in parsed_args.network:
-            network = utils.find_resource(
-                client.networks,
-                network,
-            )
-            client.networks.delete(network.id)
+        network = utils.find_resource(client.networks, self.r)
+        client.networks.delete(network.id)
 
 
 class ListNetwork(common.NetworkAndComputeLister):
     """List networks"""
 
     def update_parser_common(self, parser):
-        parser.add_argument(
+        router_ext_group = parser.add_mutually_exclusive_group()
+        router_ext_group.add_argument(
             '--external',
             action='store_true',
-            default=False,
-            help='List external networks',
+            help=_("List external networks")
+        )
+        router_ext_group.add_argument(
+            '--internal',
+            action='store_true',
+            help=_("List internal networks")
         )
         parser.add_argument(
             '--long',
             action='store_true',
-            default=False,
-            help='List additional fields in output',
+            help=_("List additional fields in output")
+        )
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            help=_("List networks according to their name")
+        )
+        admin_state_group = parser.add_mutually_exclusive_group()
+        admin_state_group.add_argument(
+            '--enable',
+            action='store_true',
+            help=_("List enabled networks")
+        )
+        admin_state_group.add_argument(
+            '--disable',
+            action='store_true',
+            help=_("List disabled networks")
+        )
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_("List networks according to their project (name or ID)")
+        )
+        identity_common.add_project_domain_option_to_parser(parser)
+        shared_group = parser.add_mutually_exclusive_group()
+        shared_group.add_argument(
+            '--share',
+            action='store_true',
+            help=_("List networks shared between projects")
+        )
+        shared_group.add_argument(
+            '--no-share',
+            action='store_true',
+            help=_("List networks not shared between projects")
+        )
+        parser.add_argument(
+            '--status',
+            metavar='<status>',
+            choices=['ACTIVE', 'BUILD', 'DOWN', 'ERROR'],
+            help=_("List networks according to their status "
+                   "('ACTIVE', 'BUILD', 'DOWN', 'ERROR')")
         )
         return parser
 
     def take_action_network(self, client, parsed_args):
+        identity_client = self.app.client_manager.identity
         if parsed_args.long:
             columns = (
                 'id',
@@ -290,7 +374,7 @@ class ListNetwork(common.NetworkAndComputeLister):
                 'shared',
                 'subnets',
                 'provider_network_type',
-                'router_external',
+                'router:external',
                 'availability_zones',
             )
             column_headers = (
@@ -317,10 +401,36 @@ class ListNetwork(common.NetworkAndComputeLister):
                 'Subnets',
             )
 
+        args = {}
+
         if parsed_args.external:
-            args = {'router:external': True}
-        else:
-            args = {}
+            args['router:external'] = True
+        elif parsed_args.internal:
+            args['router:external'] = False
+
+        if parsed_args.name is not None:
+            args['name'] = parsed_args.name
+
+        if parsed_args.enable:
+            args['admin_state_up'] = True
+        elif parsed_args.disable:
+            args['admin_state_up'] = False
+
+        if parsed_args.project:
+            project = identity_common.find_project(
+                identity_client,
+                parsed_args.project,
+                parsed_args.project_domain,
+            )
+            args['tenant_id'] = project.id
+
+        if parsed_args.share:
+            args['shared'] = True
+        elif parsed_args.no_share:
+            args['shared'] = False
+
+        if parsed_args.status:
+            args['status'] = parsed_args.status
 
         data = client.networks(**args)
 
@@ -390,6 +500,24 @@ class SetNetwork(command.Command):
             action='store_true',
             help=_("Do not share the network between projects")
         )
+        parser.add_argument(
+            '--description',
+            metavar="<description",
+            help=_("Set network description")
+        )
+        port_security_group = parser.add_mutually_exclusive_group()
+        port_security_group.add_argument(
+            '--enable-port-security',
+            action='store_true',
+            help=_("Enable port security by default for ports created on "
+                   "this network")
+        )
+        port_security_group.add_argument(
+            '--disable-port-security',
+            action='store_true',
+            help=_("Disable port security by default for ports created on "
+                   "this network")
+        )
         external_router_grp = parser.add_mutually_exclusive_group()
         external_router_grp.add_argument(
             '--external',
@@ -413,7 +541,7 @@ class SetNetwork(command.Command):
             action='store_true',
             help=_("Do not use the network as the default external network")
         )
-        _add_provider_network_options(parser)
+        _add_additional_network_options(parser)
         return parser
 
     def take_action(self, parsed_args):
@@ -421,10 +549,6 @@ class SetNetwork(command.Command):
         obj = client.find_network(parsed_args.network, ignore_missing=False)
 
         attrs = _get_attrs(self.app.client_manager, parsed_args)
-        if attrs == {}:
-            msg = "Nothing specified to be set"
-            raise exceptions.CommandError(msg)
-
         client.update_network(obj, **attrs)
 
 
